@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: SOAP::Test.pm,v 0.51 2001/07/18 15:15:14 $
+# $Id: Test.pm,v 1.3 2003/08/11 05:50:47 paulclinger Exp $
 #
 # ======================================================================
 
@@ -12,7 +12,7 @@ package SOAP::Test;
 
 use 5.004;
 use vars qw($VERSION $TIMEOUT);
-$VERSION = '0.51';
+$VERSION = sprintf("%d.%s", map {s/_//g; $_} q$Name: release-0_60-public $ =~ /-(\d+)_([\d_]+)/);
 
 $TIMEOUT = 5;
 
@@ -58,16 +58,15 @@ sub run_for {
   }
   # ------------------------------------------------------
 
-  plan tests => 49;
+  plan tests => 53;
 
   eval q!use SOAP::Lite on_fault => sub{ref $_[1] ? $_[1] : new SOAP::SOM}; 1! or die;
 
-  print "Perl SOAP server test(s)...\n";
+  print STDERR "Perl SOAP server test(s)...\n";
 
   $s = SOAP::Lite
-    -> uri('urn:/My/Examples')                
-    -> proxy($proxy)
-  ;
+    -> uri('urn:/My/Examples')
+      -> proxy($proxy);
 
   ok($s->getStateName(1)->result eq 'Alabama'); 
   ok($s->getStateNames(1,4,6,13)->result =~ /^Alabama\s+Arkansas\s+Colorado\s+Illinois\s*$/); 
@@ -85,7 +84,7 @@ sub run_for {
     $s->autoresult($autoresult);
   }
 
-  print "Autobinding of output parameters test(s)...\n";
+  print STDERR "Autobinding of output parameters test(s)...\n";
 
   $s->uri('urn:/My/Parameters');
   my $param1 = 10;
@@ -93,26 +92,32 @@ sub run_for {
   my $result = $s->autobind($param1, $param2)->result;
   ok($result == $param1 && $param2->value == 24); 
 
-  print "Header manipulation test(s)...\n";
+  print STDERR "Header manipulation test(s)...\n";
   $a = $s->addheader(2, SOAP::Header->name(my => 123)); 
   ok(ref $a->header && $a->header->{my} eq '123123'); 
   ok($a->headers eq '123123'); 
 
-  print "Echo untyped data test(s)...\n";
+  print STDERR "Echo untyped data test(s)...\n";
   $a = $s->echotwo(11, 12);
   ok($a->result == 11); 
 
-  print "mustUnderstand test(s)...\n";
+  print STDERR "mustUnderstand test(s)...\n";
   $s->echo(SOAP::Header->name(somethingelse => 123)
                        ->mustUnderstand(1));
-  ok($s->call->faultstring =~ /Header has mustUnderstand attribute/);
+  ok($s->call->faultstring =~ /[Hh]eader has mustUnderstand attribute/);
+
+  if ($proxy =~ /^http/) {
+    ok($s->transport->status =~ /^500/);
+  } else {
+    skip('No Status checks for non http protocols on server side' => undef);
+  }
 
   $s->echo(SOAP::Header->name(somethingelse => 123)
                        ->mustUnderstand(1)
                        ->actor('http://notme/'));
   ok(!defined $s->call->fault);
 
-  print "dispatch_from test(s)...\n";
+  print STDERR "dispatch_from test(s)...\n";
   eval "use SOAP::Lite
     uri => 'http://my.own.site.com/My/Examples',
     dispatch_from => ['A', 'B'],
@@ -134,7 +139,7 @@ sub run_for {
   eval { A->a };
   ok(!$@ && SOAP::Lite->self->call->faultstring =~ /Failed to access class \(A\)/);
 
-  print "Object autobinding and SOAP:: prefix test(s)...\n";
+  print STDERR "Object autobinding and SOAP:: prefix test(s)...\n";
 
   eval "use SOAP::Lite +autodispatch =>
     uri => 'urn:', proxy => '$proxy'; 1" or die;
@@ -163,7 +168,7 @@ sub run_for {
   $p = $s->SOAP::new(10); 
   ok(ref $p && $s->SOAP::next($p)+1 == $p->value);
 
-  print "VersionMismatch test(s)...\n";
+  print STDERR "VersionMismatch test(s)...\n";
 
   {
     local $SOAP::Constants::NS_ENV = 'http://schemas.xmlsoap.org/new/envelope/';
@@ -176,12 +181,12 @@ sub run_for {
     ok(ref $r && $r->faultcode =~ /:VersionMismatch/);
   }
 
-  print "Objects-by-reference test(s)...\n";
+  print STDERR "Objects-by-reference test(s)...\n";
 
   eval "use SOAP::Lite +autodispatch =>
     uri => 'urn:', proxy => '$proxy'; 1" or die;
 
-  print "Session iterator\n";
+  print STDERR "Session iterator\n";
   $r = My::SessionIterator->new(10); 
   if (!ref $r || exists $r->{id}) {
     ok(ref $r && $r->next && $r->next == 11);
@@ -189,7 +194,7 @@ sub run_for {
     skip('No persistent objects (o-b-r) supported on server side' => undef);
   }
 
-  print "Persistent iterator\n";
+  print STDERR "Persistent iterator\n";
   $r = My::PersistentIterator->new(10); 
   if (!ref $r || exists $r->{id}) {
     my $first = ($r->next, $r->next) if ref $r;   
@@ -201,8 +206,8 @@ sub run_for {
   }
 
   { local $^W; # disable warnings about deprecated AUTOLOADing for nonmethods
-    print "Parameters-by-name test(s)...\n";
-    print "You can see warning about AUTOLOAD for non-method...\n" if $^W;
+    print STDERR "Parameters-by-name test(s)...\n";
+    print STDERR "You can see warning about AUTOLOAD for non-method...\n" if $^W;
 
     eval "use SOAP::Lite +autodispatch => 
       uri => 'http://my.own.site.com/My/Parameters', proxy => '$proxy'; 1" or die;
@@ -216,12 +221,16 @@ sub run_for {
     # switch to 'main' package, because nonqualified methods should be there
     ok(main::byname(@parameters) eq "a=111, b=222, c=333");
 
-    print "Function call test(s)...\n";
-    print "You can see warning about AUTOLOAD for non-method...\n" if $^W;
+    ok(main::bynameororder(@parameters) eq "a=111, b=222, c=333");
+
+    ok(main::bynameororder(111, 222, 333) eq "a=111, b=222, c=333");
+
+    print STDERR "Function call test(s)...\n";
+    print STDERR "You can see warning about AUTOLOAD for non-method...\n" if $^W;
     ok(main::echo(11) == 11);
   }
 
-  print "SOAPAction test(s)...\n";
+  print STDERR "SOAPAction test(s)...\n";
   if ($proxy =~ /^tcp:/) {
     for (1..2) {skip('No SOAPAction checks for tcp: protocol on server side' => undef)}
   } else {
@@ -236,9 +245,26 @@ sub run_for {
     ok($s->getStateName(1)->faultstring =~ /SOAPAction shall match/); 
   }
 
+  print STDERR "UTF8 test(s)...\n";
+  if (!eval "pack('U*', 0)") {
+    for (1) {skip('No UTF8 test. No support for pack("U*") modifier' => undef)}
+  } else {
+    $s = SOAP::Lite
+      -> uri('http://my.own.site.com/My/Parameters')                
+      -> proxy($proxy);
+
+     my $latin1 = 'ਢ';
+     my $utf8 = pack('U*', unpack('C*', $latin1));
+     my $result = $s->echo(SOAP::Data->type(string => $utf8))->result;
+
+     ok(pack('U*', unpack('C*', $result)) eq $utf8                       # should work where XML::Parser marks resulting strings as UTF-8
+     || join('', unpack('C*', $result)) eq join('', unpack('C*', $utf8)) # should work where it doesn't
+     );
+  }
+
   {
     my $on_fault_was_called = 0;
-    print "Die in server method test(s)...\n";
+    print STDERR "Die in server method test(s)...\n";
     my $s = SOAP::Lite
       -> uri('http://my.own.site.com/My/Parameters')                
       -> proxy($proxy)
@@ -258,7 +284,7 @@ sub run_for {
     ok($fault->{faultactor} eq 'http://www.soaplite.com/custom');
   }
 
-  print "Method with attributes test(s)...\n";
+  print STDERR "Method with attributes test(s)...\n";
 
   $s = SOAP::Lite
     -> uri('urn:/My/Examples')                
@@ -267,7 +293,7 @@ sub run_for {
 
   ok($s->call(SOAP::Data->name('getStateName')->attr({xmlns => 'urn:/My/Examples'}), 1)->result eq 'Alabama');
 
-  print "Call with empty uri test(s)...\n";
+  print STDERR "Call with empty uri test(s)...\n";
   $s = SOAP::Lite
     -> uri('')                
     -> proxy($proxy)
@@ -277,7 +303,7 @@ sub run_for {
 
   ok($s->call('a:getStateName' => 1)->faultstring =~ /Denied access to method \(getStateName\) in class \(main\)/);
 
-  print "Number of parameters test(s)...\n";
+  print STDERR "Number of parameters test(s)...\n";
 
   $s = SOAP::Lite
     -> uri('http://my.own.site.com/My/Parameters')                
@@ -287,21 +313,21 @@ sub run_for {
   { my @all = $s->echo(1)->paramsall; ok(@all == 1) }
   { my @all = $s->echo((1) x 10)->paramsall; ok(@all == 10) }
 
-  print "Memory refresh test(s)...\n";
+  print STDERR "Memory refresh test(s)...\n";
 
   # Funny test. 
   # Let's forget about ALL settings we did before with 'use SOAP::Lite...'
   SOAP::Lite->self(undef); 
   ok(!defined SOAP::Lite->self);
 
-  print "Call without uri test(s)...\n";
+  print STDERR "Call without uri test(s)...\n";
   $s = SOAP::Lite
     -> proxy($proxy)
   ;
 
   ok($s->getStateName(1)->faultstring =~ /Denied access to method \(getStateName\) in class \(main\)/);
 
-  print "Different settings for method and namespace test(s)...\n";
+  print STDERR "Different settings for method and namespace test(s)...\n";
 
   ok($s->call(SOAP::Data
     ->name('getStateName')
@@ -309,7 +335,7 @@ sub run_for {
 
   ok($s->call(SOAP::Data
     ->name('a:getStateName')
-    ->attr({'xmlns:~' => 'urn:/My/Examples'}), 1)->result eq 'Alabama');
+    ->uri('urn:/My/Examples'), 1)->result eq 'Alabama');
 
   ok($s->call(SOAP::Data
     ->name('getStateName')
@@ -330,7 +356,7 @@ sub run_for {
   eval "use SOAP::Lite 
     uri => 'urn:/My/Examples', proxy => '$proxy'; 1" or die;
 
-  print "Global settings test(s)...\n";
+  print STDERR "Global settings test(s)...\n";
   $s = new SOAP::Lite;
 
   ok($s->getStateName(1)->result eq 'Alabama');
