@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: SOAP::Transport::HTTP.pm,v 0.46 2001/01/31 16:30:24 $
+# $Id: HTTP.pm,v 1.4 2003/08/11 05:50:47 paulclinger Exp $
 #
 # ======================================================================
 
@@ -12,31 +12,33 @@ package SOAP::Transport::HTTP;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.46';
+$VERSION = sprintf("%d.%s", map {s/_//g; $_} q$Name: release-0_60-public $ =~ /-(\d+)_([\d_]+)/);
+
+use SOAP::Lite;
 
 # ======================================================================
 
 package SOAP::Transport::HTTP::Client;
 
-use vars qw(@ISA);
+use vars qw(@ISA $COMPRESS);
 @ISA = qw(SOAP::Client LWP::UserAgent);
 
-use SOAP::Lite;
+$COMPRESS = 'deflate';
 
-my(%redirect, %mpost);
+my(%redirect, %mpost, %nocompress);
 
 # hack for HTTP conection that returns Keep-Alive 
 # miscommunication (?) between LWP::Protocol and LWP::Protocol::http
-# die after timeout
-sub patch { 
-  local $^W; 
-  eval "package LWP::UserAgent; sub redirect_ok {1}";
-  { package LWP::Protocol; 
-    my $collect = \&collect; # store original  
-    *collect = sub {          
+# dies after timeout, but seems like we could make it work
+sub patch {
+  local $^W;
+  { sub LWP::UserAgent::redirect_ok; *LWP::UserAgent::redirect_ok = sub {1} }
+  { package LWP::Protocol;
+    my $collect = \&collect; # store original
+    *collect = sub {
       if (defined $_[2]->header('Connection') && $_[2]->header('Connection') eq 'Keep-Alive') {
-        my $data = $_[3]->(); 
-        my $next = length($$data) == $_[2]->header('Content-Length') ? sub { \'' } : $_[3];
+        my $data = $_[3]->();
+        my $next = SOAP::Utils::bytelength($$data) == $_[2]->header('Content-Length') ? sub { \'' } : $_[3];
         my $done = 0; $_[3] = sub { $done++ ? &$next : $data };
       }
       goto &$collect;
@@ -47,7 +49,9 @@ sub patch {
 
 sub DESTROY { SOAP::Trace::objects('()') }
 
-sub new { require LWP::UserAgent; patch;
+sub new {
+  require LWP::UserAgent;
+  patch if $SOAP::Constants::PATCH_HTTP_KEEPALIVE;
   my $self = shift;
 
   unless (ref $self) {
@@ -56,6 +60,7 @@ sub new { require LWP::UserAgent; patch;
     while (@_) { $class->can($_[0]) ? push(@methods, shift() => shift) : push(@params, shift) }
     $self = $class->SUPER::new(@params);
     $self->agent(join '/', 'SOAP::Lite', 'Perl', SOAP::Transport::HTTP->VERSION);
+    $self->options({});
     while (@methods) { my($method, $params) = splice(@methods,0,2);
       $self->$method(ref $params eq 'ARRAY' ? @$params : $params) 
     }
@@ -66,50 +71,120 @@ sub new { require LWP::UserAgent; patch;
 
 sub send_receive {
   my($self, %parameters) = @_;
-  my($envelope, $endpoint, $action) = 
-    @parameters{qw(envelope endpoint action)};
-
+  my($envelope, $endpoint, $action, $encoding, $headers) =
+    @parameters{qw(envelope endpoint action encoding headers)};
+  # MIME:                                            ^^^^^^^
+  # MIME: I modified this because the transport layer needs access to the
+  #       HTTP headers to properly set the content-type
   $endpoint ||= $self->endpoint;
 
-  my $method = 'POST';
+  my $method='POST';
+  $COMPRESS='gzip';
   my $resp;
-  while (1) { 
 
-    # check cache for redirect
-    $endpoint = $redirect{$endpoint} if exists $redirect{$endpoint};
-    # check cache for M-POST
-    $method = 'M-POST' if exists $mpost{$endpoint};
+  $self->options->{is_compress}
+    ||= exists $self->options->{compress_threshold}
+      && eval { require Compress::Zlib };
 
-    my $req = HTTP::Request->new($method => $endpoint, HTTP::Headers->new, $envelope);
-    $req->proxy_authorization_basic($ENV{'HTTP_proxy_user'}, $ENV{'HTTP_proxy_pass'})
-      if ($ENV{'HTTP_proxy_user'} && $ENV{'HTTP_proxy_pass'}); # by Murray Nesbitt 
+ COMPRESS: {
 
-    if ($method eq 'M-POST') {
-      my $prefix = sprintf '%04d', int(rand(1000));
-      $req->header(Man => qq!"$SOAP::Constants::NS_ENV"; ns=$prefix!);
-      $req->header("$prefix-SOAPAction" => $action);  
-    } else {
-      $req->header(SOAPAction => $action);
-    }
-    $req->content_type('text/xml');
-    $req->content_length(length($envelope));
+    my $compressed
+      = !exists $nocompress{$endpoint} &&
+	$self->options->{is_compress} &&
+	  ($self->options->{compress_threshold} || 0) < length $envelope;
+    $envelope = Compress::Zlib::memGzip($envelope) if $compressed;
 
-    SOAP::Trace::transport($req);
-    SOAP::Trace::debug($req->as_string);
-    
-    $self->SUPER::env_proxy if $ENV{'HTTP_proxy'};
+    while (1) {
+      # check cache for redirect
+      $endpoint = $redirect{$endpoint} if exists $redirect{$endpoint};
+      # check cache for M-POST
+      $method = 'M-POST' if exists $mpost{$endpoint};
 
-    $resp = $self->SUPER::request($req);
+      # what's this all about?
+      # unfortunately combination of LWP and Perl 5.6.1 and later has bug
+      # in sending multibyte characters. LWP uses length() to calculate
+      # content-length header and starting 5.6.1 length() calculates chars
+      # instead of bytes. 'use bytes' in THIS file doesn't work, because
+      # it's lexically scoped. Unfortunately, content-length we calculate
+      # here doesn't work either, because LWP overwrites it with
+      # content-length it calculates (which is wrong) AND uses length()
+      # during syswrite/sysread, so we are in a bad shape anyway.
 
-    SOAP::Trace::transport($resp);
-    SOAP::Trace::debug($resp->as_string);
+      # what to do? we calculate proper content-length (using
+      # bytelength() function from SOAP::Utils) and then drop utf8 mark
+      # from string (doing pack with 'C0A*' modifier) if length and
+      # bytelength are not the same
+      my $bytelength = SOAP::Utils::bytelength($envelope);
+      $envelope = pack('C0A*', $envelope) 
+        if !$SOAP::Constants::DO_NOT_USE_LWP_LENGTH_HACK && length($envelope) != $bytelength;
 
-    # 100 OK, continue to read?
-    if (($resp->code == 510 || $resp->code == 501) && 
-        $method ne 'M-POST') { 
-      $mpost{$endpoint} = 1;
-    } else {
-      last;
+      my $req =
+	HTTP::Request->new($method => $endpoint,
+			   (defined $headers ? $headers : HTTP::Headers->new),
+      # MIME:              ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+      # MIME: This is done so that the HTTP Headers instance is properly
+      #       created --BR
+			   $envelope);
+      $req->protocol('HTTP/1.1');
+
+      $req->proxy_authorization_basic($ENV{'HTTP_proxy_user'},
+				      $ENV{'HTTP_proxy_pass'})
+	if ($ENV{'HTTP_proxy_user'} && $ENV{'HTTP_proxy_pass'});
+      # by Murray Nesbitt
+
+      if ($method eq 'M-POST') {
+	my $prefix = sprintf '%04d', int(rand(1000));
+	$req->header(Man => qq!"$SOAP::Constants::NS_ENV"; ns=$prefix!);
+	$req->header("$prefix-SOAPAction" => $action) if defined $action;
+      } else {
+	$req->header(SOAPAction => $action) if defined $action;
+      }
+
+      # allow compress if present and let server know we could handle it
+      $req->header(Accept => ['text/xml', 'multipart/*']);
+
+      $req->header('Accept-Encoding' => 
+		   [$SOAP::Transport::HTTP::Client::COMPRESS])
+	if $self->options->{is_compress};
+      $req->content_encoding($SOAP::Transport::HTTP::Client::COMPRESS)
+	if $compressed;
+
+      if(!$req->content_type){
+	$req->content_type(join '; ',
+			   'text/xml',
+			   !$SOAP::Constants::DO_NOT_USE_CHARSET && $encoding ?
+			   'charset=' . lc($encoding) : ());
+      }elsif (!$SOAP::Constants::DO_NOT_USE_CHARSET && $encoding ){
+	my $tmpType=$req->headers->header('Content-type');
+	# MIME:     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+	# MIME: This was changed from $req->content_type which was a bug,
+	#       because it does not properly maintain the entire content-type
+	#       header.
+	$req->content_type($tmpType.'; charset=' . lc($encoding));
+      }
+
+      $req->content_length($bytelength);
+      SOAP::Trace::transport($req);
+      SOAP::Trace::debug($req->as_string);
+
+      $self->SUPER::env_proxy if $ENV{'HTTP_proxy'};
+
+      $resp = $self->SUPER::request($req);
+
+      SOAP::Trace::transport($resp);
+      SOAP::Trace::debug($resp->as_string);
+
+      # 100 OK, continue to read?
+      if (($resp->code == 510 || $resp->code == 501) && $method ne 'M-POST') {
+	$mpost{$endpoint} = 1;
+      } elsif ($resp->code == 415 && $compressed) { 
+	# 415 Unsupported Media Type
+	$nocompress{$endpoint} = 1;
+	$envelope = Compress::Zlib::memGunzip($envelope);
+	redo COMPRESS; # try again without compression
+      } else {
+	last;
+      }
     }
   }
 
@@ -121,20 +196,29 @@ sub send_receive {
   $self->is_success($resp->is_success);
   $self->status($resp->status_line);
 
-  join '', $resp->content_type =~ m!^multipart/! ? ($resp->headers_as_string, "\n") : '',
-           $resp->content;
+  my $content =
+    ($resp->content_encoding || '') 
+      =~ /\b$SOAP::Transport::HTTP::Client::COMPRESS\b/o &&
+	$self->options->{is_compress} ? 
+	  Compress::Zlib::memGunzip($resp->content)
+	      : ($resp->content_encoding || '') =~ /\S/
+		? die "Can't understand returned Content-Encoding (@{[$resp->content_encoding]})\n"
+		  : $resp->content;
+  $resp->content_type =~ m!^multipart/!
+    ? join("\n", $resp->headers_as_string, $content) : $content;
 }
 
 # ======================================================================
 
 package SOAP::Transport::HTTP::Server;
 
-use vars qw(@ISA);
+use vars qw(@ISA $COMPRESS);
 @ISA = qw(SOAP::Server);
 
-use SOAP::Lite;
 use URI;
 
+$COMPRESS = 'deflate';
+
 sub DESTROY { SOAP::Trace::objects('()') }
 
 sub new { require LWP::UserAgent;
@@ -144,9 +228,11 @@ sub new { require LWP::UserAgent;
     my $class = ref($self) || $self;
     $self = $class->SUPER::new(@_);
     $self->on_action(sub {
-      (my $action = shift) =~ s/^("?)(.*)\1$/$2/;
-      die "SOAPAction shall match 'uri#method' if present\n" 
-        if $action && $action ne join('#', @_) && $action ne join('/', @_);
+      (my $action = shift || '') =~ s/^("?)(.*)\1$/$2/;
+      die "SOAPAction shall match 'uri#method' if present (got '$action', expected '@{[join('#', @_)]}'\n"
+        if $action && $action ne join('#', @_) 
+                   && $action ne join('/', @_)
+                   && (substr($_[0], -1, 1) ne '/' || $action ne join('', @_));
     });
     SOAP::Trace::objects('()');
   }
@@ -168,26 +254,38 @@ sub handle {
   my $self = shift->new;
 
   if ($self->request->method eq 'POST') {
-    $self->action($self->request->header('SOAPAction'));
+    $self->action($self->request->header('SOAPAction') || undef);
   } elsif ($self->request->method eq 'M-POST') {
     return $self->response(HTTP::Response->new(510, # NOT EXTENDED
            "Expected Mandatory header with $SOAP::Constants::NS_ENV as unique URI")) 
       if $self->request->header('Man') !~ /^"$SOAP::Constants::NS_ENV";\s*ns\s*=\s*(\d+)/;
-    $self->action($self->request->header("$1-SOAPAction"));
+    $self->action($self->request->header("$1-SOAPAction") || undef);
   } else {
     return $self->response(HTTP::Response->new(405)) # METHOD NOT ALLOWED
   }
 
+  my $compressed = ($self->request->content_encoding || '') =~ /\b$COMPRESS\b/;
+  $self->options->{is_compress} ||= $compressed && eval { require Compress::Zlib };
+
+  # signal error if content-encoding is 'deflate', but we don't want it OR
+  # something else, so we don't understand it
+  return $self->response(HTTP::Response->new(415)) # UNSUPPORTED MEDIA TYPE
+    if $compressed && !$self->options->{is_compress} ||
+       !$compressed && ($self->request->content_encoding || '') =~ /\S/;
+
   my $content_type = $self->request->content_type || '';
   # in some environments (PerlEx?) content_type could be empty, so allow it also
   # anyway it'll blow up inside ::Server::handle if something wrong with message
-  # TBD: but what to do with MIME encoded messages in THESE environments?
-  return $self->make_fault($SOAP::Constants::FAULT_CLIENT, 'Bad Request' => 'Content-Type must be text/xml')
-    if $content_type && $content_type ne 'text/xml' && $content_type !~ m!^multipart/!;
+  # TBD: but what to do with MIME encoded messages in THOSE environments?
+  return $self->make_fault($SOAP::Constants::FAULT_CLIENT, "Content-Type must be 'text/xml' instead of '$content_type'")
+    if $content_type && 
+       $content_type ne 'text/xml' && 
+       $content_type !~ m!^multipart/!;
 
+  my $content = $compressed ? Compress::Zlib::uncompress($self->request->content) : $self->request->content;
   my $response = $self->SUPER::handle(
-    join '', $content_type =~ m!^multipart/! ? ($self->request->headers_as_string, "\n") : '', 
-             $self->request->content
+    $self->request->content_type =~ m!^multipart/! 
+      ? join("\n", $self->request->headers_as_string, $content) : $content
   ) or return;
 
   $self->make_response($SOAP::Constants::HTTP_ON_SUCCESS_CODE, $response);
@@ -202,17 +300,36 @@ sub make_fault {
 sub make_response {
   my $self = shift;
   my($code, $response) = @_;
-  $response =~ s!(\?>)!$1<?xml-stylesheet type="text/css"?>! if $self->request->content_type eq 'multipart/form-data';
-  $self->response(HTTP::Response->new( 
-     $code => undef, 
+
+  my $encoding = $1
+    if $response =~ /^<\?xml(?: version="1.0"| encoding="([^"]+)")+\?>/;
+  $response =~ s!(\?>)!$1<?xml-stylesheet type="text/css"?>!
+    if $self->request->content_type eq 'multipart/form-data';
+
+  $self->options->{is_compress} ||=
+    exists $self->options->{compress_threshold} && eval { require Compress::Zlib };
+
+  my $compressed = $self->options->{is_compress} &&
+    grep(/\b($COMPRESS|\*)\b/, $self->request->header('Accept-Encoding')) &&
+      ($self->options->{compress_threshold} || 0) < SOAP::Utils::bytelength $response;
+  $response = Compress::Zlib::compress($response) if $compressed;
+  my ($is_multipart) = ($response =~ /content-type:.* boundary="([^\"]*)"/im);
+  $self->response(HTTP::Response->new(
+     $code => undef,
      HTTP::Headers->new(
-       'SOAPServer' => join('/', 'SOAP::Lite', 'Perl', SOAP::Transport::HTTP->VERSION),
-       'Content-Type' => 'text/xml', 
-       'Content-Length' => length $response), 
+			'SOAPServer' => $self->product_tokens,
+			$compressed ? ('Content-Encoding' => $COMPRESS) : (),
+			'Content-Type' => join('; ', 'text/xml',
+					       !$SOAP::Constants::DO_NOT_USE_CHARSET &&
+					       $encoding ? 'charset=' . lc($encoding) : ()),
+			'Content-Length' => SOAP::Utils::bytelength $response),
      $response,
   ));
+  $self->response->headers->header('Content-Type' => 'Multipart/Related; type="text/xml"; start="<main_envelope>"; boundary="'.$is_multipart.'"') if $is_multipart;
 }
 
+sub product_tokens { join '/', 'SOAP::Lite', 'Perl', SOAP::Transport::HTTP->VERSION }
+
 # ======================================================================
 
 package SOAP::Transport::HTTP::CGI;
@@ -236,13 +353,21 @@ sub new {
 sub handle {
   my $self = shift->new;
 
-  my $content; read(STDIN,$content,$ENV{'CONTENT_LENGTH'} || 0);  
-  $self->request(HTTP::Request->new( 
-    $ENV{'REQUEST_METHOD'} || '' => $ENV{'SCRIPT_NAME'},
-    HTTP::Headers->new(map {(/^HTTP_(.+)/i ? $1 : $_) => $ENV{$_}} keys %ENV),
-    $content,
-  ));
-  $self->SUPER::handle;
+  my $length = $ENV{'CONTENT_LENGTH'} || 0;
+
+  if (!$length) {     
+    $self->response(HTTP::Response->new(411)) # LENGTH REQUIRED
+  } elsif (defined $SOAP::Constants::MAX_CONTENT_SIZE && $length > $SOAP::Constants::MAX_CONTENT_SIZE) {
+    $self->response(HTTP::Response->new(413)) # REQUEST ENTITY TOO LARGE
+  } else {
+    my $content; binmode(STDIN); read(STDIN,$content,$length);
+    $self->request(HTTP::Request->new( 
+      $ENV{'REQUEST_METHOD'} || '' => $ENV{'SCRIPT_NAME'},
+      HTTP::Headers->new(map {(/^HTTP_(.+)/i ? $1 : $_) => $ENV{$_}} keys %ENV),
+      $content,
+    ));
+    $self->SUPER::handle;
+  }
 
   # imitate nph- cgi for IIS (pointed by Murray Nesbitt)
   my $status = defined($ENV{'SERVER_SOFTWARE'}) && $ENV{'SERVER_SOFTWARE'}=~/IIS/
@@ -269,9 +394,15 @@ sub new { require HTTP::Daemon;
 
   unless (ref $self) {
     my $class = ref($self) || $self;
-    $self = $class->SUPER::new();
-    $self->{_daemon} = HTTP::Daemon->new(@_) or Carp::croak "Can't create daemon: $!";
+
+    my(@params, @methods);
+    while (@_) { $class->can($_[0]) ? push(@methods, shift() => shift) : push(@params, shift) }
+    $self = $class->SUPER::new;
+    $self->{_daemon} = HTTP::Daemon->new(@params) or Carp::croak "Can't create daemon: $!";
     $self->myuri(URI->new($self->url)->canonical->as_string);
+    while (@methods) { my($method, $params) = splice(@methods,0,2);
+      $self->$method(ref $params eq 'ARRAY' ? @$params : $params) 
+    }
     SOAP::Trace::objects('()');
   }
   return $self;
@@ -294,7 +425,9 @@ sub handle {
       $self->SUPER::handle;
       $c->send_response($self->response)
     }
-    $c->close;
+    # replaced ->close, thanks to Sean Meisner <Sean.Meisner@VerizonWireless.com>
+    # shutdown() doesn't work on AIX. close() is used in this case. Thanks to Jos Clijmans <jos.clijmans@recyfin.be>
+    UNIVERSAL::isa($c, 'shutdown') ? $c->shutdown(2) : $c->close(); 
     undef $c;
   }
 }
@@ -308,7 +441,7 @@ use vars qw(@ISA);
 
 sub DESTROY { SOAP::Trace::objects('()') }
 
-sub new { require Apache;
+sub new { require Apache; require Apache::Constants;
   my $self = shift;
 
   unless (ref $self) {
@@ -330,19 +463,72 @@ sub handler {
   ));
   $self->SUPER::handle;
 
-  if ($self->response->is_success) {
-    $r->header_out('Content-length' => $self->response->content_length);
-    $r->send_http_header($self->response->content_type);
-    $r->print($self->response->content);
-  } else {
-    $r->err_header_out('Content-length' => $self->response->content_length);
-    $r->content_type($self->response->content_type);
-    $r->custom_response($self->response->code, $self->response->content);
-  }
-  $self->response->code;
+  # we will specify status manually for Apache, because
+  # if we do it as it has to be done, returning SERVER_ERROR,
+  # Apache will modify our content_type to 'text/html; ....'
+  # which is not what we want.
+  # will emulate normal response, but with custom status code 
+  # which could also be 500.
+  $r->status($self->response->code);
+  $self->response->headers->scan(sub { $r->header_out(@_) });
+  $r->send_http_header(join '; ', $self->response->content_type);
+  $r->print($self->response->content);
+  &Apache::Constants::OK;
 }
 
-*handle = \&handler; # just create alias
+sub configure {
+  my $self = shift->new;
+  my $config = shift->dir_config;
+  foreach (%$config) {
+    $config->{$_} =~ /=>/
+      ? $self->$_({split /\s*(?:=>|,)\s*/, $config->{$_}})
+      : ref $self->$_() ? () # hm, nothing can be done here
+                        : $self->$_(split /\s+|\s*,\s*/, $config->{$_})
+      if $self->can($_);
+  }
+  $self;
+}
+
+{ sub handle; *handle = \&handler } # just create alias
+
+# ======================================================================
+#
+# Copyright (C) 2001 Single Source oy (marko.asplund@kronodoc.fi)
+# a FastCGI transport class for SOAP::Lite.
+#
+# ======================================================================
+
+package SOAP::Transport::HTTP::FCGI;
+
+use vars qw(@ISA);
+@ISA = qw(SOAP::Transport::HTTP::CGI);
+
+sub DESTROY { SOAP::Trace::objects('()') }
+
+sub new { require FCGI; Exporter::require_version('FCGI' => 0.47); # requires thread-safe interface
+  my $self = shift;
+
+  if (!ref($self)) {
+    my $class = ref($self) || $self;
+    $self = $class->SUPER::new(@_);
+    $self->{_fcgirq} = FCGI::Request(\*STDIN, \*STDOUT, \*STDERR);
+    SOAP::Trace::objects('()');
+  }
+  return $self;
+}
+
+sub handle {
+  my $self = shift->new;
+
+  my ($r1, $r2);
+  my $fcgirq = $self->{_fcgirq};
+
+  while (($r1 = $fcgirq->Accept()) >= 0) {
+    $r2 = $self->SUPER::handle;
+  }
+
+  return undef;
+}
 
 # ======================================================================
 
@@ -400,7 +586,16 @@ SOAP::Transport::HTTP - Server/Client side HTTP support for SOAP::Lite
 
 =item Apache mod_perl server
 
-See F<examples/server/Apache.pm> and L</EXAMPLES> section for more information.
+See F<examples/server/Apache.pm> and L</"EXAMPLES"> section for more information.
+
+=item mod_soap server (.htaccess, directory-based access)
+
+  SetHandler perl-script
+  PerlHandler Apache::SOAP
+  PerlSetVar dispatch_to "/Your/Path/To/Deployed/Modules, Module::Name, Module::method"
+  PerlSetVar options "compress_threshold => 10000"
+
+See L<Apache::SOAP> for more information.
 
 =back
 
@@ -481,7 +676,7 @@ handled.
 
 You can use any proxy setting you use with LWP::UserAgent modules:
 
- SOAP::Lite->proxy('http://endpoint.server', 
+ SOAP::Lite->proxy('http://endpoint.server/', 
                    proxy => ['http' => 'http://my.proxy.server']);
 
 or
@@ -492,6 +687,95 @@ should specify proxy server for you. And if you use C<HTTP_proxy_user>
 and C<HTTP_proxy_pass> for proxy authorization SOAP::Lite should know 
 how to handle it properly. 
 
+=head2 COOKIE-BASED AUTHENTICATION
+
+  use HTTP::Cookies;
+
+  my $cookies = HTTP::Cookies->new(ignore_discard => 1);
+    # you may also add 'file' if you want to keep them between sessions
+
+  my $soap = SOAP::Lite->proxy('http://localhost/');
+  $soap->transport->cookie_jar($cookies);
+
+Cookies will be taken from response and provided for request. You may
+always add another cookie (or extract what you need after response)
+with HTTP::Cookies interface.
+
+You may also do it in one line:
+
+  $soap->proxy('http://localhost/', 
+               cookie_jar => HTTP::Cookies->new(ignore_discard => 1));
+
+=head2 SSL CERTIFICATE AUTHENTICATION
+
+To get certificate authentication working you need to specify three
+environment variables: C<HTTPS_CERT_FILE>, C<HTTPS_KEY_FILE>, and 
+(optionally) C<HTTPS_CERT_PASS>:
+
+  $ENV{HTTPS_CERT_FILE} = 'client-cert.pem';
+  $ENV{HTTPS_KEY_FILE}  = 'client-key.pem';
+
+Crypt::SSLeay (which is used for https support) will take care about 
+everything else. Other options (like CA peer verification) can be specified
+in a similar way. See Crypt::SSLeay documentation for more details.
+
+Those who would like to use encrypted keys may check 
+http://groups.yahoo.com/group/soaplite/message/729 for details. 
+
+=head2 COMPRESSION
+
+SOAP::Lite provides you with the option for enabling compression on the 
+wire (for HTTP transport only). Both server and client should support 
+this capability, but this should be absolutely transparent to your 
+application. The Server will respond with an encoded message only if 
+the client can accept it (indicated by client sending an Accept-Encoding 
+header with 'deflate' or '*' values) and client has fallback logic, 
+so if server doesn't understand specified encoding 
+(Content-Encoding: deflate) and returns proper error code 
+(415 NOT ACCEPTABLE) client will repeat the same request without encoding
+and will store this server in a per-session cache, so all other requests 
+will go there without encoding.
+
+Having options on client and server side that let you specify threshold
+for compression you can safely enable this feature on both client and 
+server side.
+
+=over 4
+
+=item Client
+
+  print SOAP::Lite
+    -> uri('http://localhost/My/Parameters')
+    -> proxy('http://localhost/', options => {compress_threshold => 10000})
+    -> echo(1 x 10000)
+    -> result
+  ;
+
+=item Server
+
+  my $server = SOAP::Transport::HTTP::CGI
+    -> dispatch_to('My::Parameters')
+    -> options({compress_threshold => 10000})
+    -> handle;
+
+=back
+
+Compression will be enabled on the client side 
+B<if> the threshold is specified 
+B<and> the size of current message is bigger than the threshold 
+B<and> the module Compress::Zlib is available. 
+
+The Client will send the header 'Accept-Encoding' with value 'deflate'
+B<if> the threshold is specified 
+B<and> the module Compress::Zlib is available.
+
+Server will accept the compressed message if the module Compress::Zlib 
+is available, and will respond with the compressed message 
+B<only if> the threshold is specified 
+B<and> the size of the current message is bigger than the threshold 
+B<and> the module Compress::Zlib is available 
+B<and> the header 'Accept-Encoding' is presented in the request.
+
 =head1 EXAMPLES
 
 Consider following examples of SOAP servers:
@@ -589,7 +873,7 @@ and you are using Apache web server, try to put into your httpd.conf
      PassEnv LD_LIBRARY_PATH
  </IfModule>
 
-=item Apache is crashing with segfaults
+=item Apache is crashing with segfaults (it may looks like "500 unexpected EOF before status line seen" on client side)
 
 If using SOAP::Lite (or XML::Parser::Expat) in combination with mod_perl
 causes random segmentation faults in httpd processes try to configure
@@ -597,10 +881,22 @@ Apache with:
 
  RULE_EXPAT=no
 
+-- OR (for Apache 1.3.20 and later) --
+
+ ./configure --disable-rule=EXPAT
+
 See http://archive.covalent.net/modperl/2000/04/0185.xml for more 
-details and lot of thanks to Robert Barta (rho@bigpond.net.au) for
+details and lot of thanks to Robert Barta <rho@bigpond.net.au> for
 explaining this weird behavior.
 
+If it doesn't help, you may also try -Uusemymalloc
+(or something like that) to get perl to use the system's own malloc.
+Thanks to Tim Bunce <Tim.Bunce@pobox.com>.
+
+=item CGI scripts are not running under Microsoft Internet Information Server (IIS)
+
+CGI scripts may not work under IIS unless scripts are .pl, not .cgi.
+
 =back
 
 =head1 DEPENDENCIES
