@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: JABBER.pm 341 2009-09-30 18:34:58Z kutterma $
+# $Id: JABBER.pm,v 1.1.1.1 2002/11/01 14:53:56 paulclinger Exp $
 #
 # ======================================================================
 
@@ -12,12 +12,12 @@ package SOAP::Transport::JABBER;
 
 use strict;
 use vars qw($VERSION);
-
-use SOAP::Lite; $VERSION = $SOAP::Lite::VERSION;
+$VERSION = sprintf("%d.%s", map {s/_//g; $_} q$Name: release-0_60-public $ =~ /-(\d+)_([\d_]+)/);
 
 use Net::Jabber 1.0021 qw(Client); 
 use URI::Escape; 
 use URI;
+use SOAP::Lite;
 
 my $NAMESPACE = "http://namespaces.soaplite.com/transport/jabber";
 
@@ -242,3 +242,53 @@ sub handle {
 1;
 
 __END__
+
+=head1 NAME
+
+SOAP::Transport::JABBER - Server/Client side JABBER support for SOAP::Lite
+
+=head1 SYNOPSIS
+
+=over 4
+
+=item Client
+
+  use SOAP::Lite 
+    uri => 'http://my.own.site.com/My/Examples',
+    proxy => 'jabber://username:password@jabber.org:5222/soaplite_server@jabber.org/',
+    #         proto    username passwd   server     port destination                resource (optional)
+  ;
+
+  print getStateName(1);
+
+=item Server
+
+  use SOAP::Transport::JABBER;
+
+  my $server = SOAP::Transport::JABBER::Server
+    -> new('jabber://username:password@jabber.org:5222')
+    # specify list of objects-by-reference here 
+    -> objects_by_reference(qw(My::PersistentIterator My::SessionIterator My::Chat))
+    # specify path to My/Examples.pm here
+    -> dispatch_to('/Your/Path/To/Deployed/Modules', 'Module::Name', 'Module::method')
+  ;
+
+  print "Contact to SOAP server\n";
+  do { $server->handle } while sleep 10;
+
+=back
+
+=head1 DESCRIPTION
+
+=head1 COPYRIGHT
+
+Copyright (C) 2000-2001 Paul Kulchenko. All rights reserved.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+=head1 AUTHOR
+
+Paul Kulchenko (paulclinger@yahoo.com)
+
+=cut
